@@ -1098,8 +1098,9 @@ void
 convertStringToAbacus(AbacusWidget w, const char *string, int aux)
 {
 	int decimal = 0;
-	int num, factor, i, sign = 0, len = (int) strlen(string);
+	int digit, factor, i, sign = 0, len = (int) strlen(string);
 	AbacusWidget *wap = &w, wa;
+	int topFactor, bottomNumber, diff, base;
 
 	if (aux == 1) {
 		if (w->abacus.leftAux == NULL)
@@ -1111,6 +1112,10 @@ convertStringToAbacus(AbacusWidget w, const char *string, int aux)
 		wap = (AbacusWidget *) w->abacus.rightAux;
 	}
 	wa = *wap;
+	topFactor = wa->abacus.decks[TOP].factor;
+	bottomNumber = wa->abacus.decks[BOTTOM].number;
+	diff = bottomNumber - topFactor;
+	base = wa->abacus.base;
 	if (aux == 0) {
 		clearRails(w);
 #ifndef WINVER
@@ -1125,17 +1130,24 @@ convertStringToAbacus(AbacusWidget w, const char *string, int aux)
 	}
 	for (i = 0; i < decimal - sign; i++) {
 		/* wa->abacus.displayBase == wa->abacus.base and all that ... */
-		num = char2Int(string[decimal - 1 - i]);
-		if (wa->abacus.decks[BOTTOM].number <= wa->abacus.base / 2) {
-			factor = num / wa->abacus.decks[TOP].factor;
-			if (factor > 0) {
-				setAbacusMove(w, ACTION_MOVE, aux, 1, i, factor);
-				num = num - factor * wa->abacus.decks[TOP].factor;
+		digit = char2Int(string[decimal - 1 - i]);
+		if (bottomNumber < base - 1 &&
+				base % topFactor != 0 && diff >= 0 &&
+				digit >= base - 1 - diff) { /* Cn16 */
+			int topNumber = (base - 1) / topFactor - 1;
+			setAbacusMove(w, ACTION_MOVE, aux, 1, i, topNumber);
+			setAbacusMove(w, ACTION_MOVE, aux, 0, i, digit % topFactor + topFactor);
+		} else {
+			if (bottomNumber < base - 1) {
+				factor = digit / topFactor;
+				if (factor > 0) {
+					setAbacusMove(w, ACTION_MOVE, aux, 1, i, factor);
+					digit = digit - factor * topFactor;
+				}
+			}
+			if (digit > 0) {
+				setAbacusMove(w, ACTION_MOVE, aux, 0, i, digit);
 			}
-		}
-		factor = num / wa->abacus.decks[BOTTOM].factor;
-		if (factor > 0) {
-			setAbacusMove(w, ACTION_MOVE, aux, 0, i, factor);
 		}
 	}
 	if (wa->abacus.sign && string[0] == '-') {
@@ -1145,23 +1157,32 @@ convertStringToAbacus(AbacusWidget w, const char *string, int aux)
 	for (i = 0; i < len - decimal - 1 &&
 			i < wa->abacus.decimalPosition; i++) {
 		int offset = 0;
+		int j;
 
-		num = char2Int(string[decimal + i + 1]);
+		digit = char2Int(string[decimal + i + 1]);
 		if  (wa->abacus.decks[BOTTOM].piece != 0)
 			offset++;
 		if  (wa->abacus.decks[BOTTOM].piecePercent != 0 &&
 				i >= wa->abacus.shiftPercent)
 			offset++;
-		if (wa->abacus.decks[BOTTOM].number <= wa->abacus.base / 2) {
-			factor = num / wa->abacus.decks[TOP].factor;
-			if (factor > 0) {
-				setAbacusMove(w, ACTION_MOVE, aux, 1, -i - 1 - offset, factor);
-				num = num - factor * wa->abacus.decks[TOP].factor;
+		j = -i - 1 - offset;
+		if (bottomNumber < base - 1 &&
+				base % topFactor != 0 && diff >= 0 &&
+				digit >= base - 1 - diff) { /* Cn16 */
+			int topNumber = (base - 1) / topFactor - 1;
+			setAbacusMove(w, ACTION_MOVE, aux, 1, j, topNumber);
+			setAbacusMove(w, ACTION_MOVE, aux, 0, j, digit % topFactor + topFactor);
+		} else {
+			if (bottomNumber < base - 1) {
+				factor = digit / topFactor;
+				if (factor > 0) {
+					setAbacusMove(w, ACTION_MOVE, aux, 1, j, factor);
+					digit = digit - factor * topFactor;
+				}
+			}
+			if (digit > 0) {
+				setAbacusMove(w, ACTION_MOVE, aux, 0, j, digit);
 			}
-		}
-		factor = num / wa->abacus.decks[BOTTOM].factor;
-		if (factor > 0) {
-			setAbacusMove(w, ACTION_MOVE, aux, 0, -i - 1 - offset, factor);
 		}
 	}
 }
