@@ -1,7 +1,7 @@
 /*
  * @(#)AbacusD.c
  *
- * Copyright 1999 - 2019  David A. Bagley, bagleyd AT verizon.net
+ * Copyright 1999 - 2020  David A. Bagley, bagleyd AT verizon.net
  *
  * Abacus demo and neat pointers from
  * Copyright 1991 - 1998  Luis Fernandes, elf AT ee.ryerson.ca
@@ -664,8 +664,8 @@ doLesson(AbacusWidget w)
 			} else {
 				tmp = abacusLibrary.book[w->abacusDemo.bookCount].edition[edition].chapter[w->abacusDemo.chapterCount].lesson[w->abacusDemo.lessonCount].move[w->abacusDemo.moveCount].lineText[line]; /* 64 char */
 			}
-			(void) strncpy(displayText[line], tmp, strlen(tmp) + 1);
-			displayText[line][strlen(displayText[line])] = '\0';
+			(void) strncpy(displayText[line], tmp, CHARS - 1);
+			displayText[line][CHARS - 1] = '\0';
 		}
 	}
 
