@@ -392,13 +392,15 @@ extern int string2Roman(char *buf, char *inbuf, int base, Boolean decimalFractio
 extern void string2Group(char *buf, char *inbuf, int groupSize,
 	char decimalPoint, char groupSeparator);
 extern void calculate(AbacusWidget w, char *buffer, int aux);
-extern void teachStep(AbacusWidget w, char *buffer, int aux);
+extern void teachStep(AbacusWidget w, const char *buffer, int aux);
 extern LONG multPower(int m, int x, int n);
 extern Boolean checkPiece(AbacusWidget w);
 extern Boolean checkPiecePercent(AbacusWidget w);
 extern Boolean checkSubdeck(AbacusWidget w, int position);
 extern void drawBeadRail(AbacusWidget w, int rail, Boolean highlight);
 extern int convertBaseToBottom(int base);
+extern int divInt(int a, int b);
+extern int modInt(int a, int b);
 extern int rootInt(int i, int n);
 
 extern void newMoves(AbacusStack *s);
